/*  graph library based on array lists
    (#vertices and #edges can not be increased after the initialization
            12/Feb/2002   by Takeaki Uno  e-mail:uno@nii.jp, 
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, please 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about the codes for the users. */


#ifndef _agraph_h_
#define _agraph_h_

#include"alist.h"

/*  return the index of the internal index of an edge e incident to vertex v */
#define AGRAPH_EDGE(G,e)   ((e)%(G).edge_end)
/*  for an edge e(given by internal index), return the vertex incident to e such that e deos not belong to its incident list*/
#define AGRAPH_INC(G,e)    ((G).edge.list[((e)+(G).edge_end)%((G).edge_end*2)])
/*   return the opposite internal index of an edge e of internal index  */
#define AGRAPH_OPO(G,e)    (((e)+(G).edge_end)%((G).edge_end*2))
/*  For edge e and vertex v incident to e, return the other vertex incident to e  */
#define AGRAPH_ADJ(G,v,e)  ((G).edge.list[(e)]==v?(G).edge.list[((e)+(G).edge_end)%((G).edge_end*2)]:(G).edge.list[e])
/* return the vertex which is the head/tail of arc a */
#define AGRAPH_TO(G,a)     ((G).in.list[(a)])
#define AGRAPH_FROM(G,a)     ((G).out.list[(a)])

/* return edge list according to f: 0:edge, 1:from, -1:to */
#define AGRAPH_EDGES(G,f)    ((f)>0?&((G).out):(f)<0?&((G).in):&((G).edge))
#define AGRAPH_INC_FROM(G,a,f)   ((f)>0?((G).out.list[(a)]):(f)<0?((G).in.list[(a)]):((G).edge.list[(a)]))
#define AGRAPH_INC_TO(G,a,f)   ((f)>0?((G).in.list[(a)]):(f)<0?((G).out.list[(a)]):((G).edge.list[((a)+(G).edge_end)%((G).edge_end*2)]))

/* the degree of vertex v: G->edge.num[v] */
#define AGRAPH_DEG(G,v)       ((G).edge.num[v])
/* the in-degree of the vertex v: G->in.num[v] */
#define AGRAPH_IN_DEG(G,v)    ((G).in.num[v])
/* the out-degree of the vertex v: G->out.num[v] */
#define AGRAPH_OUT_DEG(G,v)   ((G).out.num[v])

#define AGRAPH_EDGE_FLOOP(G,v,e) for((e)=(G).edge.nxt[(G).edge.end+v];(e)<(G).edge.end;(e)=(G).edge.nxt[(e)])

/*  structure for graph  */
typedef struct {
  unsigned char type;   // type
  char *fname;      // input file name
  int flag;         // flag for load routine
  ALIST_ID node_end;     // maximum number of vertices
  ALIST_ID edge_end;     // maximum number of edges
  ALIST_ID arc_end;      // maximum number of arcs
  ALIST_ID node_num;     // #current vertices
  ALIST_ID edge_num;     // #current edges
  ALIST_ID arc_num;      // #current arcs
  MALIST edge;     // list of edges incident to each vertex
  MALIST in;       // list of out-going arcs incident to each vertex
  MALIST out;      // list of in-coming arcs incident to each vertex
  ALIST_ID node1_num; // the size of vertex set one, which means the bipartite case (usual graph if 0)
  WEIGHT *node_w; // array for node weights
  WEIGHT *edge_w; // array for edge weights
  WEIGHT *arc_w;  // array for arc weights
} AGRAPH;

extern AGRAPH INIT_AGRAPH;

/************** initializations and terminations *************/

/*  initialization, termination, allocate arrays for weights, copy and duplication */
void AGRAPH_alloc (AGRAPH *G, ALIST_ID node_num, ALIST_ID edge_num, ALIST_ID arc_num);
void AGRAPH_alloc_weight (AGRAPH *G);
void AGRAPH_cpy (AGRAPH *G2, AGRAPH *G);
void AGRAPH_end (AGRAPH *G);

/********************* addition/deletion ****************************/

/*  initialization/take/remove/make edge e as connecting vertices u and v, 
   deleting deleted edge, making edges already made, out of range parameters, 
    are illigal use. Maybe segmentation fault occurs */
void AGRAPH_edge_init (AGRAPH *G, ALIST_ID e, ALIST_ID u, ALIST_ID v, int undo);
void AGRAPH_edge_out (AGRAPH *G, ALIST_ID e, int undo);
void AGRAPH_edge_rm (AGRAPH *G, ALIST_ID e, int undo);
ALIST_ID AGRAPH_edge_mk (AGRAPH *G, ALIST_ID u, ALIST_ID v, int undo);

/*  initialization arc a as connecting vertices u and v, take, remove,
 and make edge (u,v). 
   error if delete deleted edge, make arc already made, out of range parameters */
void AGRAPH_arc_init (AGRAPH *G, ALIST_ID a, ALIST_ID from, ALIST_ID to, int undo);
void AGRAPH_arc_out (AGRAPH *G, ALIST_ID a, int undo);
void AGRAPH_arc_rm (AGRAPH *G, ALIST_ID a, int undo);
ALIST_ID AGRAPH_arc_mk (AGRAPH *G, ALIST_ID from, ALIST_ID to, int undo);





/* remove/recover a node from the graph */
void AGRAPH_edge_out (AGRAPH *G, ALIST_ID e, int undo);
void AGRAPH_edge_recov (AGRAPH *G, ALIST_ID e, int undo);
void AGRAPH_arc_out (AGRAPH *G, ALIST_ID a, int undo);
void AGRAPH_arc_recov (AGRAPH *G, ALIST_ID a, int undo);
void AGRAPH_node_rm (AGRAPH *G, ALIST_ID v, int undo );
void AGRAPH_node_recov (AGRAPH *G, ALIST_ID v, int undo);

/* A Function to return the maximum degree of the graph */
ALIST_ID AGRAPH_max_degree (AGRAPH *G);
ALIST_ID *AGRAPH_degree_sort_iter (AGRAPH *G, ALIST_ID s, ALIST_ID t, ALIST_ID *p, int flag);

/* make an array of vertices sorted in the order of degrees.
   Increasing order if flag==1, and decreasing order if flag ==-1 */
PERM *AGRAPH_degree_sort (AGRAPH *G, int flag);

/* For each vertex, sort the edges incident to the vertex by the IDs of
   the vertices they are adjacent to. */
void AGRAPH_sort_incident_edges (AGRAPH *G);

/* Return -1 if edge (v,u) does not exist. If exist, then return its
 working ID of (v->u) */
ALIST_ID AGRAPH_is_edge (AGRAPH *G, ALIST_ID v, ALIST_ID u);

/* Check whether graph G1 is isomorphic to G2 by permutating vertex set of 
 G1 by p. If they are isomorphic, return 0 */
int AGRAPH_equal_perm (AGRAPH *G1, AGRAPH *G2, ALIST_ID *p);


/* replace multiple edges and multiple arcs by single one */
void AGRAPH_simple (AGRAPH *G);


/******************* print routines *************************************/

/*  print graph by numbers  */
void AGRAPH_print (AGRAPH *G);

/******************* search algorithms ******************************/

/* for bipartite graph with un-known separater n1, s.t., vertices<n1 are the
 vertex set 1, find n1. If the graph is non-bipartite, return -1 */
ALIST_ID AGRAPH_find_node1_num (AGRAPH *G);

/* A Function to return the maximum degree of the graph */
ALIST_ID AGRAPH_max_degree (AGRAPH *G);

/* random graph generators
  flag&1 = 0: undirected, 1: directed
  flag&2 = 0: don't allow self loop, 2: allow self loop
  flag&4 = 0: don't allow bidirected arc, 4: allow bidirected arc
  flag&8 = 0: genral graph, 8: bipartite graph
    use all vertices from 0 to n-1 */

/* generating randam graph part 1. flag%16 ==0 => the probability of the 
 existence of each edge is per/1000, o.w., #edges is just "per" */
/* node1_num >0 => generate bipartite graph */
void AGRAPH_mk_rand_graph1 (AGRAPH *G, int per, int flag);

/* random graph generator 2, the degree of each vertex is roughly close 
 to the specified "deg", but it works well only in the case that n is 
  large, such as over 100. #edges never exceed deg*node_end/2 */
/* node1_num >0 => generate bipartite graph */
void AGRAPH_mk_rand_graph2 (AGRAPH *G, ALIST_ID deg, int flag);


/*************************************************************************/



/* Write the graph to file. Edges, arcs, and nodes from 0 to node_num/edge_num/arc_num are written to file. Parameters are
  (graph) (file name) (not write edge weight => 0) (not write node weight => 0) */
void AGRAPH_save (AGRAPH *G, char *fname);


/* graph load routine. Allocate memory as much as the size of input file.
   parameters are, 
   (graph) (file name) (read edges?) (read arcs?) (read node weights?) (read edge weight?) (bipartite?) */
void AGRAPH_load (AGRAPH *G);

/*
  format of file:(including notifications to make input file)
   
  the ith row corresponds to node i-1, and
    ID list of nodes adjacent to i, and having ID > i, for undirected graph
    ID list of nodes adjacent to i by out-going arc of i, for directed graph
   Separator is ",", but graph load routine accepts any letter for 
    separator but not a number.
   If the graph has both edges and arcs, write them in two lines separately,
    so a node then uses two lines, and #nodes = #lines/2.
  
    ==  Notifications to make input file ==
   Notice that if 0th line has node 2, and the 2nd line has 0, then there
    will be multiple edge (0,2) and (2,0).
   The read routine does not make error with multiple edges, it is allowed.

   The ID of nodes begin from 0. After reading graph, node_num is set to
    node_end.

   Input file example, without weights, E={(0,1),(0,2),(1,1),(1,3),(2,3)}
===========
   1,2
   1 3
   3
   
   [EOF]
=========
   Nodes are 0,1, and 2, both edges and arcs exist, with node/edge/arc weights)
   5000,1,30
   0,50,1,20,
   100,1,3
   2,20
   200
   
   [EOF]
=======
   where node weights are 5000, 100, 200, and edges and their weights are
    (0,1),30,   (1,1),3
    arcs and their weights are (0,0),50,   (0,1), 20,   (1,2), 20

    In the case of bipartite graph, write the adjacent-node lists only for 
     the node in node set one.
     
    
*/
#endif 








